#ifndef HSPM_H
#define HSPM_H

#include <Eigen/Dense>
#include <utils.h>
#include <vector>

using namespace std;

class HSPM {
public:
    void generateNaca4DigitCoordinates(double camber, double camberPos, double thickness, size_t _N);
    void loadCoordinates(char* fileName);
    void saveCoordinates(char* fileName);
    void initHSPM();
    void computeConstantInfluenceCoeffs();
    Eigen::MatrixXd getSpeedAtPoint(double x_p, double y_p);
    void imposeBlowingVelocity(size_t i, double _blVel);
    void setdStar(size_t i, double _dStar);
    void init();
    void solve();
    double solveOffBodyKutta();
    void computeInviscidVelocity();
    void computePressureDistribution();
    void saveCp(char* fileName);

    size_t N;
    double chord;
    Eigen::VectorXd x;
    Eigen::VectorXd y;
    Eigen::VectorXd x_m;
    Eigen::VectorXd y_m;
    Eigen::VectorXd blVel;
    Eigen::VectorXd U;
    Eigen::VectorXd V;
    Eigen::VectorXd dStar;
    Eigen::VectorXd theta;
    Eigen::MatrixXd r;
    Eigen::MatrixXd beta;
    Eigen::VectorXd lengths;
    double l;
    Eigen::MatrixXd A_n;
    Eigen::MatrixXd A_t;
    Eigen::MatrixXd B_n;
    Eigen::MatrixXd B_t;

    double V_inf;
    double V_x;
    double V_y;
    double AoA;

    Eigen::VectorXd b;
    Eigen::VectorXd c;

    double tau;
    Eigen::VectorXd s1;
    Eigen::VectorXd s2;
    Eigen::VectorXd q;
    Eigen::MatrixXd Cp;
    Eigen::VectorXd Vt;
    double cd;
    double cl;
    double cm;

    // Wake stuff
    double L_wake;
    size_t N_wake;
    Eigen::VectorXd x_wake;
    Eigen::VectorXd y_wake;
    Eigen::VectorXd x_m_wake;
    Eigen::VectorXd y_m_wake;
    Eigen::VectorXd U_wake;
    Eigen::VectorXd V_wake;

    // Mesh stuff
    enum MeshType {
    CosN = 0,
    ConstantN = 1,
    Cos2N = 2,
    LinearS = 3,
    ExponentialS = 4
    };
    MeshType meshType;
    double sLe, sTe;
    double prog; // For exponential mesh

    bool cutTE;
    bool stopTE;
    double stopTELocation;

private:
    Eigen::PartialPivLU<Eigen::MatrixXd> solver;
};

#endif